@@ -15,14 +15,14 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 #include <tqlayout.h>
 
-#include <klocale.h>
-#include <kglobal.h>
-#include <kfontdialog.h>
+#include <tdelocale.h>
+#include <tdeglobal.h>
+#include <tdefontdialog.h>
 #include <kiconloader.h>
 #include <tqfile.h>
 #include <tqtextstream.h>
@@ -35,19 +35,19 @@
 #include <tqradiobutton.h>
 #include <tqlabel.h>
 #include <kurllabel.h>
-#include <kapplication.h>
-#include <kfontcombo.h>
+#include <tdeapplication.h>
+#include <tdefontcombo.h>
 #include <tqspinbox.h>
 #include <ksqueezedtextlabel.h>
 #include <stdlib.h>
-#include <kmessagebox.h>
-#include <kconfig.h>
+#include <tdemessagebox.h>
+#include <tdeconfig.h>
 #include <tqstyle.h>
 #include <tqheader.h>
-#include <klistview.h>
+#include <tdelistview.h>
 #include <tqmessagebox.h>
 #include <tqcheckbox.h>
-#include <kglobal.h>
+#include <tdeglobal.h>
 #include <kstandarddirs.h>
 
 #include <sys/types.h>
@@ -57,17 +57,20 @@
 #include "mozillaprofile.h"
 
 const TQString KcmGtk::GTK_RC_FILE(".gtkrc-2.0-kde-kde4");
-const TQString KcmGtk::KDE_RC_DIR(KGlobal::dirs()->localkdedir() + "/env/");
+const TQString KcmGtk::KDE_RC_DIR(TDEGlobal::dirs()->localtdedir() + "/env/");
 const TQString KcmGtk::GTK_TQT_RC_FILE("gtk-qt-engine.rc.sh");
 
+const TQString KcmGtk::GTK3_INI_DIR(TQDir::homeDirPath() + "/.config/gtk-3.0/");
+const TQString KcmGtk::GTK3_INI_FILE("settings.ini");
+
 /*typedef KGenericFactory<KcmGtk, TQWidget> KcmGtkFactory;
 K_EXPORT_COMPONENT_FACTORY( kcm_gtk, KcmGtkFactory("gtk"))*/
 
 extern "C"
 {
-	KCModule *create_kcmgtk( TQWidget * parent, const char * name )
+	TDECModule *create_kcmgtk( TQWidget * parent, const char * name )
 	{
-		KGlobal::locale()->insertCatalogue( "gtkqtengine" );
+		TDEGlobal::locale()->insertCatalogue( "gtkqtengine" );
 		return new KcmGtk( parent, "kcmgtk" );
 	}
 }
@@ -157,14 +160,14 @@ TQFont GtkRcParser::parseFont(TQString fontString)
 }
 
 KcmGtk::KcmGtk(TQWidget *parent, const char *name, const TQStringList&)
-    : KCModule(parent, name),
+    : TDECModule(parent, name),
       myAboutData(0),
       emacsDetailsDialog(NULL),
       searchPathsDialog(NULL)
 {
-	KGlobal::locale()->insertCatalogue("gtkqtengine");
+	TDEGlobal::locale()->insertCatalogue("gtkqtengine");
 	
-	config = new KConfig("kcmgtkrc");
+	config = new TDEConfig("kcmgtkrc");
 
 	TQStringList gtkSearchPathsDefault;
 	gtkSearchPathsDefault.append("/usr");
@@ -181,26 +184,30 @@ KcmGtk::KcmGtk(TQWidget *parent, const char *name, const TQStringList&)
 	l->addWidget(widget);
 	
 	// Load the icons
-	KIconLoader iconLoader;
-	widget->styleIcon->setPixmap(iconLoader.loadIcon("style", KIcon::Desktop));
-	widget->fontIcon->setPixmap(iconLoader.loadIcon("fonts", KIcon::Desktop));
-	widget->firefoxIcon->setPixmap(iconLoader.loadIcon("firefox", KIcon::Desktop));
-	widget->keyboardIcon->setPixmap(iconLoader.loadIcon("keyboard", KIcon::Desktop));
+	TDEIconLoader iconLoader;
+	widget->styleIcon->setPixmap(iconLoader.loadIcon("style", TDEIcon::Desktop));
+	widget->styleIcon3->setPixmap(iconLoader.loadIcon("style", TDEIcon::Desktop));
+	widget->fontIcon->setPixmap(iconLoader.loadIcon("fonts", TDEIcon::Desktop));
+	widget->keyboardIcon->setPixmap(iconLoader.loadIcon("input-keyboard", TDEIcon::Desktop));
 	
 	getInstalledThemes();
 	load();
 	
 	// Connect some signals
-	connect(widget->warning2, TQT_SIGNAL(leftClickedURL(const TQString&)), KApplication::kApplication(), TQT_SLOT(invokeBrowser(const TQString&)));
+	connect(widget->warning2, TQT_SIGNAL(leftClickedURL(const TQString&)), TDEApplication::kApplication(), TQT_SLOT(invokeBrowser(const TQString&)));
 	connect(widget->styleGroup, TQT_SIGNAL(clicked(int)), TQT_SLOT(itemChanged()));
+	connect(widget->warning5, TQT_SIGNAL(leftClickedURL(const TQString&)), TDEApplication::kApplication(), TQT_SLOT(invokeBrowser(const TQString&)));
+	connect(widget->styleGroup3, TQT_SIGNAL(clicked(int)), TQT_SLOT(itemChanged()));
 	connect(widget->fontGroup, TQT_SIGNAL(clicked(int)), TQT_SLOT(itemChanged()));
 	connect(widget->styleBox, TQT_SIGNAL(activated(int)), TQT_SLOT(itemChanged()));
 	connect(widget->styleBox, TQT_SIGNAL(activated(int)), TQT_SLOT(styleChanged()));
+	connect(widget->styleBox3, TQT_SIGNAL(activated(int)), TQT_SLOT(itemChanged()));
+	connect(widget->styleBox3, TQT_SIGNAL(activated(int)), TQT_SLOT(styleChanged()));
 	connect(widget->emacsBox, TQT_SIGNAL(toggled(bool)), TQT_SLOT(itemChanged()));
 	connect(widget->fontChange, TQT_SIGNAL(clicked()), TQT_SLOT(fontChangeClicked()));
-	connect(widget->firefoxFix, TQT_SIGNAL(clicked()), TQT_SLOT(firefoxFixClicked()));
 	connect(widget->emacsDetails, TQT_SIGNAL(clicked()), TQT_SLOT(emacsDetailsClicked()));
 	connect(widget->warning3, TQT_SIGNAL(clicked()), TQT_SLOT(searchPathsClicked()));
+	connect(widget->warning6, TQT_SIGNAL(clicked()), TQT_SLOT(searchPathsClicked()));
 }
 
 void KcmGtk::getInstalledThemes()
@@ -223,14 +230,61 @@ void KcmGtk::getInstalledThemes()
 		}
 	}
 	
+	bool installed = false;
 	widget->styleBox->clear();
-	widget->styleBox->insertStringList(themes.keys());
-	
-	bool installed = (themes.find("Qt") != themes.end());
+	if(!themes.empty()) {
+		TQStringList otherStyles = themes.keys();
+		if(otherStyles.find("Qt")!=otherStyles.end()) {
+			otherStyles.remove(otherStyles.find("Qt"));
+		}
+		installed = !otherStyles.empty();
+		if(installed) {
+			widget->styleBox->insertStringList(otherStyles);
+		}
+	}
 	widget->styleKde->setEnabled(installed);
+	widget->styleBox->setEnabled(installed);
+	widget->styleOther->setEnabled(installed);
 	widget->warning1->setHidden(installed);
 	widget->warning2->setHidden(installed);
 	widget->warning3->setHidden(installed);
+
+	gtk3Themes.clear();
+	for ( TQStringList::Iterator it = gtkSearchPaths.begin(); it != gtkSearchPaths.end(); ++it )
+	{
+		TQString path = (*it) + "/share/themes/";
+		TQDir dir(path);
+		TQStringList entryList = dir.entryList(TQDir::Dirs, TQDir::Unsorted);
+		for ( TQStringList::Iterator it2 = entryList.begin(); it2 != entryList.end(); ++it2 )
+		{
+			if ((*it2).startsWith("."))
+				continue;
+			if (gtk3Themes.find(*it2) != gtk3Themes.end())
+				continue;
+			if (!TQFile::exists(path + (*it2) + "/gtk-3.0/gtk.css"))
+				continue;
+			gtk3Themes.insert((*it2), path + (*it2) + "/gtk-3.0/gtk.css");
+		}
+	}
+	
+	bool gtk3installed = !gtk3Themes.empty();
+	widget->styleBox3->clear();
+	if(gtk3installed) {
+		TQStringList otherGtk3Styles = gtk3Themes.keys();
+		if(otherGtk3Styles.find("tdegtk")!=otherGtk3Styles.end()) {
+			otherGtk3Styles.remove(otherGtk3Styles.find("tdegtk"));
+		}
+		if(otherGtk3Styles.find("Default")==otherGtk3Styles.end()) {
+			otherGtk3Styles.prepend("Default");
+		}
+		widget->styleBox3->insertStringList(otherGtk3Styles);
+	}
+	widget->styleKde3->setEnabled(gtk3installed);
+	widget->styleBox3->setEnabled(gtk3installed);
+	widget->styleOther3->setEnabled(gtk3installed);
+	widget->warning4->setHidden(gtk3installed);
+	widget->warning5->setHidden(gtk3installed);
+	widget->warning6->setHidden(gtk3installed);
 }
 
 void KcmGtk::itemChanged()
@@ -241,7 +295,7 @@ void KcmGtk::itemChanged()
 
 void KcmGtk::fontChangeClicked()
 {
-	if ( KFontDialog::getFont( font ) == KFontDialog::Accepted )
+	if ( TDEFontDialog::getFont( font ) == TDEFontDialog::Accepted )
 	{
 		updateFontPreview();
 		widget->fontGroup->setButton(widget->fontGroup->id(widget->fontOther));
@@ -306,10 +360,12 @@ void KcmGtk::load()
 			break;
 		}
 		
-		if (usingQtEngine)
+		if (usingQtEngine) {
 			widget->styleGroup->setButton(widget->styleGroup->id(widget->styleKde));
-		else
+		}
+		else {
 			widget->styleGroup->setButton(widget->styleGroup->id(widget->styleOther));
+		}
 	}
 
 	font = parser.font;
@@ -324,6 +380,30 @@ void KcmGtk::load()
 	widget->emacsBox->setChecked(parser.emacs);
 	
 	updateFontPreview();
+
+	bool usingGtk3TQtEngine = false;
+	if (TQFile::exists(GTK3_INI_DIR + GTK3_INI_FILE)) {
+		TQString activeGtk3Style;
+		TDEConfig gtk3Config(GTK3_INI_DIR + GTK3_INI_FILE);
+		gtk3Config.setGroup("Settings");
+		activeGtk3Style = gtk3Config.readEntry("gtk-theme-name");
+		if (activeGtk3Style == "tdegtk") {
+			usingGtk3TQtEngine = true;
+		}
+
+		if (usingGtk3TQtEngine) {
+			widget->styleGroup3->setButton(widget->styleGroup3->id(widget->styleKde3));
+		}
+		else {
+			widget->styleGroup3->setButton(widget->styleGroup3->id(widget->styleOther3));
+			for (int i=0; i<widget->styleBox3->count(); ++i) {
+				if (widget->styleBox3->text(i) == activeGtk3Style) {
+					widget->styleBox3->setCurrentItem(i);
+					break;
+				}
+			}
+		}
+	}
 }
 
 
@@ -342,9 +422,12 @@ void KcmGtk::save()
 	
 	TQString themeName = widget->styleKde->isChecked() ? themes["Qt"] : themes[widget->styleBox->currentText()];
 	TQString themeNameShort = widget->styleKde->isChecked() ? TQString("Qt") : widget->styleBox->currentText();
+
+	TQString themeName3 = widget->styleKde3->isChecked() ? gtk3Themes["tdegtk"] : gtk3Themes[widget->styleBox3->currentText()];
+	TQString themeNameShort3 = widget->styleKde3->isChecked() ? TQString("tdegtk") : widget->styleBox3->currentText();
 	
-	stream << "# This file was written by KDE\n";
-	stream << "# You can edit it in the KDE control center, under \"GTK Styles and Fonts\"\n";
+	stream << "# This file was written by TDE\n";
+	stream << "# You can edit it in the TDE control center, under \"GTK Styles and Fonts\"\n";
 	stream << "\n";
 	stream << "include \"" << themeName << "\"\n";
 	if (TQFile::exists("/etc/gtk-2.0/gtkrc"))
@@ -404,7 +487,7 @@ void KcmGtk::save()
 	
 	// Tell the user to restart KDE if the environment file was created this time
 	if (envFileDidNotExist)
-		TQMessageBox::information(this, "Restart KDE", "Your changes have been saved, but you will have to restart KDE for them to take effect.", TQMessageBox::Ok);
+		TQMessageBox::information(this, "Restart TDE", "Your changes have been saved, but you will have to restart TDE for them to take effect.", TQMessageBox::Ok);
 	
 	// Older versions of the Gtk-Qt theme engine wrote directly into ~/.gtkrc-2.0
 	// If the user has upgraded, that file needs to be deleted so the old settings
@@ -417,7 +500,7 @@ void KcmGtk::save()
 		file.readLine(firstLine, 50);
 		file.close();
 		
-		if (firstLine == "# This file was written by KDE")
+		if (firstLine == "# This file was written by TDE")
 			file.remove();
 	}
 	
@@ -438,203 +521,38 @@ void KcmGtk::save()
 		stream << fileDataString;
 		file.close();
 	}
-	
+
+	TQDir gtk3IniDir;
+	if (!gtk3IniDir.exists(GTK3_INI_DIR))
+	{
+		// Make sure the GTK INI dir exists
+		gtk3IniDir.mkdir(GTK3_INI_DIR);
+	}
+
+	TQFile gtk3IniFile(GTK3_INI_DIR + GTK3_INI_FILE);
+	gtk3IniFile.open(IO_WriteOnly);
+	TQTextStream gtk3IniStream(&gtk3IniFile);
+	gtk3IniStream << "# This file was written by TDE\n";
+	gtk3IniStream << "# You can edit it in the TDE control center, under \"GTK Styles and Fonts\"\n";
+	gtk3IniStream << "\n";
+	gtk3IniStream << "[Settings]\n";
+	gtk3IniStream << "gtk-theme-name = " << themeNameShort3 << "\n";
+	gtk3IniStream << "gtk-icon-theme-name = tdegtk-icon-theme\n";
+	gtk3IniStream << "gtk-fallback-icon-theme = tdegtk-fallback-icon-theme\n";
+	gtk3IniFile.close();
+
 	emit changed(true);
 }
 
 
 int KcmGtk::buttons()
 {
-	return KCModule::Apply;
+	return TDECModule::Apply;
 }
 
 TQString KcmGtk::quickHelp() const
 {
-	return i18n("");
-}
-
-
-void KcmGtk::firefoxFixClicked()
-{
-	profiles.clear();
-	getProfiles(TQDir::homeDirPath() + "/.mozilla/firefox/", 0);
-	getProfiles(TQDir::homeDirPath() + "/.thunderbird/", 1);
-	
-	TQString profilePath;
-	if (profiles.count() == 0)
-	{
-		KMessageBox::error(this, i18n("No Mozilla profiles found"), i18n("Could not load Mozilla profiles"));
-		return;
-	}
-	else if (profiles.count() == 1)
-	{
-		fixProfile(profiles.begin().data());
-	}
-	else
-	{
-		KDialogBase* dialog = new KDialogBase(this, "", true, i18n("Mozilla profile"), KDialogBase::Ok | KDialogBase::Cancel);
-		MozillaProfileWidget* w = new MozillaProfileWidget(dialog);
-		w->profilesList->header()->hide();
-		w->profilesList->hideColumn(1);
-		
-		TQPixmap icon = KGlobal::iconLoader()->loadIcon("kuser", KIcon::Small);
-		
-		for ( TQMapIterator<TQString,TQString> it = profiles.begin(); it != profiles.end(); ++it )
-		{
-			KListViewItem* i = new KListViewItem(w->profilesList);
-			i->setPixmap(0, icon);
-			i->setText(0, it.key());
-			i->setText(1, it.data());
-		}
-		
-		dialog->setMainWidget(w);
-		if (dialog->exec() == TQDialog::Rejected)
-		{
-			delete dialog;
-			return;
-		}
-		
-		TQListViewItemIterator it2(w->profilesList, TQListViewItemIterator::Selected);
-		while (it2.current())
-		{
-			KListViewItem* i = (KListViewItem*) it2.current();
-			++it2;
-			
-			fixProfile(i->text(1));
-		}
-		delete dialog;
-	}
-	
-	KMessageBox::information(this, i18n("Your Mozilla profile was updated sucessfully.  You must close and restart all Firefox and Thunderbird windows for the changes to take effect"), i18n("Mozilla profile"));
-}
-
-void KcmGtk::getProfiles(const TQString& basePath, int type)
-{
-	TQString fileName = basePath + "/profiles.ini";
-	if (TQFile::exists(fileName))
-	{
-		KConfig config(fileName, true, false);
-		TQStringList groups = config.groupList();
-		
-		for ( TQStringList::Iterator it = groups.begin(); it != groups.end(); ++it )
-		{
-			if (!(*it).lower().startsWith("profile"))
-				continue;
-			
-			config.setGroup(*it);
-			TQString name = (type ? i18n("Thunderbird") : i18n("Firefox")) + " - " + config.readEntry("Name");
-			TQString path = config.readEntry("Path");
-			if (!path.startsWith("/"))
-				path = basePath + path;
-			profiles.insert(name, path);
-		}
-	}
-}
-
-void KcmGtk::fixProfile(const TQString& path)
-{
-	if (!TQFile::exists(path + "/chrome"))
-	{
-		TQDir dir(path);
-		dir.mkdir("chrome");
-	}
-	
-	TQString data = scrollBarCSS();
-	writeFirefoxCSS(path + "/chrome/userChrome.css", data);
-	writeFirefoxCSS(path + "/chrome/userContent.css", data);
-}
-
-TQString KcmGtk::scrollBarCSS()
-{
-	// The following code determines how many buttons are on a scrollbar
-	// It works by looking at each pixel of the scrollbar's area not taken up by the groove,
-	// and asking the style which subcontrol is at that location.
-	TQScrollBar sbar(NULL);
-	sbar.setOrientation(Qt::Horizontal);
-	sbar.setValue(1);
-	sbar.resize(200,25);
-	
-	TQRect rect = tqApp->style().querySubControlMetrics(TQStyle::CC_ScrollBar, &sbar, TQStyle::SC_ScrollBarGroove);
-	
-	bool back1 = false;
-	bool forward1 = false;
-	bool back2 = false;
-	bool forward2 = false;
-	
-	TQStyle::SubControl sc = TQStyle::SC_None;
-	for (TQPoint pos(0,7) ; pos.x()<rect.x() ; pos.setX(pos.x()+1))
-	{
-		TQStyle::SubControl sc2 = tqApp->style().querySubControl(TQStyle::CC_ScrollBar, &sbar, pos);
-		if (sc != sc2)
-		{
-			if (sc2 == TQStyle::SC_ScrollBarAddLine) forward1 = true;
-			if (sc2 == TQStyle::SC_ScrollBarSubLine) back1 = true;
-			sc = sc2;
-		}
-	}
-	sc = TQStyle::SC_None;
-	for (TQPoint pos(rect.x()+rect.width(),7) ; pos.x()<200 ; pos.setX(pos.x()+1))
-	{
-		TQStyle::SubControl sc2 = tqApp->style().querySubControl(TQStyle::CC_ScrollBar, &sbar, pos);
-		if (sc != sc2)
-		{
-			if (sc2 == TQStyle::SC_ScrollBarAddLine) forward2 = true;
-			if (sc2 == TQStyle::SC_ScrollBarSubLine) back2 = true;
-			sc = sc2;
-		}
-	}
-	
-	TQString upTop = (back1 ? "-moz-box" : "none");
-	TQString downTop = (forward1 ? "-moz-box" : "none");
-	TQString upBottom = (back2 ? "-moz-box" : "none");
-	TQString downBottom = (forward2 ? "-moz-box" : "none");
-	
-	TQString data;
-	data += "/* The following four lines were added by KDE */\n";
-	data += "scrollbarbutton[sbattr=\"scrollbar-up-top\"] { display: " + upTop + " !important; }\n";
-	data += "scrollbarbutton[sbattr=\"scrollbar-down-top\"] { display: " + downTop + " !important; }\n";
-	data += "scrollbarbutton[sbattr=\"scrollbar-up-bottom\"] { display: " + upBottom + " !important; }\n";
-	data += "scrollbarbutton[sbattr=\"scrollbar-down-bottom\"] { display: " + downBottom + " !important; }\n";
-	
-	return data;
-}
-
-void KcmGtk::writeFirefoxCSS(const TQString& path, const TQString& data)
-{
-	TQString fileData;
-	TQFile file(path);
-	if (file.open(IO_ReadOnly))
-	{
-		TQTextStream stream(&file);
-		for (;;)
-		{
-			TQString line = stream.readLine();
-			if (line.isNull())
-				break;
-			
-			if ((line == "# The following four lines were added by KDE") ||
-			    (line == "/* The following four lines were added by KDE */"))
-			{
-				for (int i=0 ; i<4 ; i++)
-					stream.readLine();
-				continue;
-			}
-			
-			fileData += line + "\n";
-		}
-		file.close();
-	}
-	
-	if (!file.open(IO_WriteOnly | IO_Truncate))
-	{
-		KMessageBox::error(this, i18n("Could not write to %1").arg(path), i18n("Mozilla profile"));
-		return;
-	}
-	TQTextStream stream(&file);
-	stream << fileData << data;
-	file.close();
-	
-	return;
+	return "";
 }
 
 void KcmGtk::emacsDetailsClicked()
